//---------------------------------------------------------------------------- 
// * Action Default Initialization {action_init_default}
//---------------------------------------------------------------------------- 
//	Default initialization of an action with no ID or owner process. The
//	name of this blank action is N/A, or 'Not Available' 
//---------------------------------------------------------------------------- 
Action::Action(){
	act_name = "N/A";
	process = 0;
}

//---------------------------------------------------------------------------- 
// * Action Initialization {action_init_proc_id}
//---------------------------------------------------------------------------- 
//	Initializes an action with an action ID of id for some process
//	referred to by the node pointer process_p. The action ID specificies
//	the name, predicate, and effect of this action.
//---------------------------------------------------------------------------- 
Action::Action(Node* process_p, unsigned short int id){
	process = process_p;
	action_id = id;

	switch(action_id){
	case 0:
		act_name = "request";
		break;
	case 1:
		act_name = "clear";
		break;
	case 2:
		act_name = "forward";
		break;
	case 3:
		act_name = "back";
		break;
	case 4: 
		act_name = "stop";
		break;
	case 5:
		act_name = "reflect";
		break;
	case 6:
		act_name = "reset";
		break;
	default:
		act_name = "N/A";
	}
}

//---------------------------------------------------------------------------- 
// * Get Action Name {action_name}
//---------------------------------------------------------------------------- 
//	Returns the action's name as a string.
//---------------------------------------------------------------------------- 
std::string Action::name() const{
	return act_name;
}

//---------------------------------------------------------------------------- 
// * Get Action Guard State {action_guard}
//---------------------------------------------------------------------------- 
//	Returns the evaluation of the action's guard predicate. Automatically
//	returns false if the action is not assigned to a process. 
//---------------------------------------------------------------------------- 
bool Action::guard() const{
	bool status = false;

	if(process > 0 && process->rooted()){
		switch(action_id){
		case 0:
			status = process->idle() && process->children_are(Idle);
			break;
		case 1:
			status = process->requesting() && process->children_are(Replying);
			break;
		case 2:
			status = process->parent()->requesting() && process->idle() && process->children_are(Idle);
			break;
		case 3:
			status = process->parent()->requesting() && process->requesting() && process->children_are(Replying);
			break;
		case 4:
			status = process->parent()->idle() && !(process->idle());
			break;
		case 5:
			status = process->parent()->requesting() && process->idle();
			break;
		case 6:
			status = process->parent()->idle() && process->replying();
			break;
		default:
			status = false;
		}
	}
	else{
		status = false;
	}

	return status;
}

//---------------------------------------------------------------------------- 
// * Get Action ID {action_id}
//---------------------------------------------------------------------------- 
//	Returns the ID of the action which specifies its name, guard
//	predicate, and execution.
//---------------------------------------------------------------------------- 
unsigned short int Action::id() const{
	return action_id;
}

//---------------------------------------------------------------------------- 
// * Execute Action {action_execute}
//---------------------------------------------------------------------------- 
//	Executes the action on the process it is assigned to. If the action is
//	not assigned to a process, nothing is done executed.
//---------------------------------------------------------------------------- 
void Action::execute() const{
	switch(action_id){
	case 0:
		process->set_state(Requesting);
		break;
	case 1:
		process->set_state(Idle);
		break;
	case 2:
		process->set_state(Requesting);
		break;
	case 3:
		process->set_state(Replying);
		break;
	case 4:
		process->set_state(Idle);
		break;
	case 5:
		process->set_state(Replying);
		break;
	case 6:
		process->set_state(Idle);
		break;
	}
}
