//--------------------------------------------------------
// * Node
//--------------------------------------------------------
#ifndef TREE_NODE_H
#define TREE_NODE_H

#include <vector>
#include <cstdlib>
#include <string>
#include "action.h"

enum states{Requesting, Idle, Replying};

class Node{
	friend class Action;

public:
	Node();
	Node(int);
	Node(int, states);
	Node(const Node&);
	~Node();

	int id() const;
	bool linked() const;
	bool root() const;
	bool intermediate() const;
	bool leaf() const;
	bool rooted() const;
	bool requesting() const;
	bool idle() const;
	bool replying() const;
	states get_state() const;
	std::string state_str() const;
	int get_causality() const;
	int actions_performed() const;
	bool neighboring(const Node*) const;	

	Node* parent();
	const Node* parent() const;
	std::vector<Node*>::iterator links_begin();
	std::vector<const Node*>::const_iterator links_begin() const;
	std::vector<Node*>::iterator links_end();
	std::vector<const Node*>::const_iterator links_end() const;
	std::vector<Node*>::iterator children_begin();
	std::vector<const Node*>::const_iterator children_begin() const;
	std::vector<Node*>::iterator children_end();
	std::vector<const Node*>::const_iterator children_end() const;

	void add_link(Node*);
	void add_child(Node*);
	void set_parent(Node*);
	void set_state(states);
	states perturb();
	void set_causality(int);
	void reset_actions();
	void disown();
	void define_actions();
	void update_causality();

	const Action* enabled() const;
	const Action* execute();

	Node* operator=(const Node&);

private:
	int data_id;
	Node* data_parent;
	std::vector<Node*> links;
	std::vector<const Node*> links_const;
	std::vector<Node*> children;
	std::vector<const Node*> children_const;

	states state;
	int actions_counter;
	int causality;
	std::vector<const Action*> actions;

	bool children_are(states) const;
};

#include "node.cpp"
#include "action.cpp"

#endif
